@@ -1,4 +1,4 @@
-#! /usr/bin/python3.3
+#! /usr/bin/python2.7
 
 """Tool for measuring execution time of small code snippets.
 
@@ -9,15 +9,14 @@ the Python Cookbook, published by O'Reilly.
 Library usage: see the Timer class.
 
 Command line usage:
-    python timeit.py [-n N] [-r N] [-s S] [-t] [-c] [-p] [-h] [--] [statement]
+    python timeit.py [-n N] [-r N] [-s S] [-t] [-c] [-h] [--] [statement]
 
 Options:
   -n/--number N: how many times to execute 'statement' (default: see below)
   -r/--repeat N: how many times to repeat the timer (default 3)
   -s/--setup S: statement to be executed once initially (default 'pass')
-  -p/--process: use time.process_time() (default is time.perf_counter())
-  -t/--time: use time.time() (deprecated)
-  -c/--clock: use time.clock() (deprecated)
+  -t/--time: use time.time() (default on Unix)
+  -c/--clock: use time.clock() (default on Windows)
   -v/--verbose: print raw timing results; repeat for more digits precision
   -h/--help: print this usage message and exit
   --: separate options from statement, use when statement starts with -
@@ -67,17 +66,23 @@ __all__ = ["Timer"]
 dummy_src_name = "<timeit-src>"
 default_number = 1000000
 default_repeat = 3
-default_timer = time.perf_counter
+
+if sys.platform == "win32":
+    # On Windows, the best timer is time.clock()
+    default_timer = time.clock
+else:
+    # On most other platforms the best timer is time.time()
+    default_timer = time.time
 
 # Don't change the indentation of the template; the reindent() calls
 # in Timer.__init__() depend on setup being indented 4 spaces and stmt
 # being indented 8 spaces.
 template = """
 def inner(_it, _timer):
-    {setup}
+    %(setup)s
     _t0 = _timer()
     for _i in _it:
-        {stmt}
+        %(stmt)s
     _t1 = _timer()
     return _t1 - _t0
 """
@@ -117,27 +122,33 @@ class Timer:
         """Constructor.  See class doc string."""
         self.timer = timer
         ns = {}
-        if isinstance(stmt, str):
+        if isinstance(stmt, basestring):
+            # Check that the code can be compiled outside a function
+            if isinstance(setup, basestring):
+                compile(setup, dummy_src_name, "exec")
+                compile(setup + '\n' + stmt, dummy_src_name, "exec")
+            else:
+                compile(stmt, dummy_src_name, "exec")
             stmt = reindent(stmt, 8)
-            if isinstance(setup, str):
+            if isinstance(setup, basestring):
                 setup = reindent(setup, 4)
-                src = template.format(stmt=stmt, setup=setup)
-            elif callable(setup):
-                src = template.format(stmt=stmt, setup='_setup()')
+                src = template % {'stmt': stmt, 'setup': setup}
+            elif hasattr(setup, '__call__'):
+                src = template % {'stmt': stmt, 'setup': '_setup()'}
                 ns['_setup'] = setup
             else:
                 raise ValueError("setup is neither a string nor callable")
             self.src = src # Save for traceback display
             code = compile(src, dummy_src_name, "exec")
-            exec(code, globals(), ns)
+            exec code in globals(), ns
             self.inner = ns["inner"]
-        elif callable(stmt):
+        elif hasattr(stmt, '__call__'):
             self.src = None
-            if isinstance(setup, str):
+            if isinstance(setup, basestring):
                 _setup = setup
                 def setup():
-                    exec(_setup, globals(), ns)
-            elif not callable(setup):
+                    exec _setup in globals(), ns
+            elif not hasattr(setup, '__call__'):
                 raise ValueError("setup is neither a string nor callable")
             self.inner = _template_func(setup, stmt)
         else:
@@ -229,7 +240,7 @@ def repeat(stmt="pass", setup="pass", timer=default_timer,
     """Convenience function to create Timer object and call repeat method."""
     return Timer(stmt, setup, timer).repeat(repeat, number)
 
-def main(args=None, *, _wrap_timer=None):
+def main(args=None, _wrap_timer=None):
     """Main program, used when run as a script.
 
     The optional 'args' argument specifies the command line to be parsed,
@@ -250,13 +261,12 @@ def main(args=None, *, _wrap_timer=None):
         args = sys.argv[1:]
     import getopt
     try:
-        opts, args = getopt.getopt(args, "n:s:r:tcpvh",
+        opts, args = getopt.getopt(args, "n:s:r:tcvh",
                                    ["number=", "setup=", "repeat=",
-                                    "time", "clock", "process",
-                                    "verbose", "help"])
-    except getopt.error as err:
-        print(err)
-        print("use -h/--help for command line help")
+                                    "time", "clock", "verbose", "help"])
+    except getopt.error, err:
+        print err
+        print "use -h/--help for command line help"
         return 2
     timer = default_timer
     stmt = "\n".join(args) or "pass"
@@ -278,14 +288,12 @@ def main(args=None, *, _wrap_timer=None):
             timer = time.time
         if o in ("-c", "--clock"):
             timer = time.clock
-        if o in ("-p", "--process"):
-            timer = time.process_time
         if o in ("-v", "--verbose"):
             if verbose:
                 precision += 1
             verbose += 1
         if o in ("-h", "--help"):
-            print(__doc__, end=' ')
+            print __doc__,
             return 0
     setup = "\n".join(setup) or "pass"
     # Include the current directory, so that local imports work (sys.path
@@ -306,7 +314,7 @@ def main(args=None, *, _wrap_timer=None):
                 t.print_exc()
                 return 1
             if verbose:
-                print("%d loops -> %.*g secs" % (number, precision, x))
+                print "%d loops -> %.*g secs" % (number, precision, x)
             if x >= 0.2:
                 break
     try:
@@ -316,18 +324,18 @@ def main(args=None, *, _wrap_timer=None):
         return 1
     best = min(r)
     if verbose:
-        print("raw times:", " ".join(["%.*g" % (precision, x) for x in r]))
-    print("%d loops," % number, end=' ')
+        print "raw times:", " ".join(["%.*g" % (precision, x) for x in r])
+    print "%d loops," % number,
     usec = best * 1e6 / number
     if usec < 1000:
-        print("best of %d: %.*g usec per loop" % (repeat, precision, usec))
+        print "best of %d: %.*g usec per loop" % (repeat, precision, usec)
     else:
         msec = usec / 1000
         if msec < 1000:
-            print("best of %d: %.*g msec per loop" % (repeat, precision, msec))
+            print "best of %d: %.*g msec per loop" % (repeat, precision, msec)
         else:
             sec = msec / 1000
-            print("best of %d: %.*g sec per loop" % (repeat, precision, sec))
+            print "best of %d: %.*g sec per loop" % (repeat, precision, sec)
     return None
 
 if __name__ == "__main__":
