@@ -50,8 +50,8 @@ build_time_vars = {'ABIFLAGS': 'm',
  'CONFINCLUDEDIR': '/usr/include',
  'CONFINCLUDEPY': '/usr/include/python3.4m',
  'COREPYTHONPATH': ':plat-x86_64-linux-gnu',
- 'COVERAGE_INFO': '/build/buildd/python3.4-3.4.2/build-shared/coverage.info',
- 'COVERAGE_REPORT': '/build/buildd/python3.4-3.4.2/build-shared/lcov-report',
+ 'COVERAGE_INFO': '/build/buildd/python3.4-3.4.3/build-shared/coverage.info',
+ 'COVERAGE_REPORT': '/build/buildd/python3.4-3.4.3/build-shared/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--no-branch-coverage --title "CPython lcov '
                             'report"',
  'CPPFLAGS': '-I. -IInclude -I../Include -D_FORTIFY_SOURCE=2',
@@ -183,6 +183,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_GCC_UINT128_T': 1,
  'HAVE_GETADDRINFO': 1,
  'HAVE_GETC_UNLOCKED': 1,
+ 'HAVE_GETENTROPY': 0,
  'HAVE_GETGROUPLIST': 1,
  'HAVE_GETGROUPS': 1,
  'HAVE_GETHOSTBYNAME': 0,
@@ -296,6 +297,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_PTY_H': 1,
  'HAVE_PUTENV': 1,
  'HAVE_PWRITE': 1,
+ 'HAVE_RAND_EGD': 1,
  'HAVE_READLINK': 1,
  'HAVE_READLINKAT': 1,
  'HAVE_READV': 1,
@@ -599,7 +601,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'READELF': 'readelf',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/build/buildd/python3.4-3.4.2/build-shared',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/build/buildd/python3.4-3.4.3/build-shared',
  'SCRIPTDIR': '/usr/lib',
  'SETPGRP_HAVE_ARG': 0,
  'SGI_ABI': '',
@@ -641,10 +643,10 @@ build_time_vars = {'ABIFLAGS': 'm',
  'TCLTK_LIBS': '',
  'TESTOPTS': '',
  'TESTPATH': '',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/build/buildd/python3.4-3.4.2/build-shared '
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/build/buildd/python3.4-3.4.3/build-shared '
                './python',
  'TESTPYTHONOPTS': '',
- 'TESTRUNNER': 'LD_LIBRARY_PATH=/build/buildd/python3.4-3.4.2/build-shared '
+ 'TESTRUNNER': 'LD_LIBRARY_PATH=/build/buildd/python3.4-3.4.3/build-shared '
                './python ../Tools/scripts/run_tests.py',
  'TESTTIMEOUT': 3600,
  'THREADOBJ': 'Python/thread.o',
@@ -670,8 +672,8 @@ build_time_vars = {'ABIFLAGS': 'm',
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/build/buildd/python3.4-3.4.2/build-shared',
- 'abs_srcdir': '/build/buildd/python3.4-3.4.2/build-shared/..',
+ 'abs_builddir': '/build/buildd/python3.4-3.4.3/build-shared',
+ 'abs_srcdir': '/build/buildd/python3.4-3.4.3/build-shared/..',
  'datarootdir': '/usr/share',
  'exec_prefix': '/usr',
  'prefix': '/usr',
